from op5helpers import *

nacoma_api_endpoint = 'https://'+os.environ.get("MONITOR_SERVER",'localhost')+'/api/config/'

def test_can_login_via_nacoma():
    r = requests.get(nacoma_api_endpoint+'host', auth=('monitor', 'monitor'), verify=False)
    eq_(r.status_code, 200)

def test_no_connection_without_auth():
    r = requests.get(nacoma_api_endpoint, verify=False)
    eq_(r.status_code, 401)

def test_no_connection_with_bad_auth():
    r = requests.get(nacoma_api_endpoint, auth=('zumth1ng', 'non-exizt1ng'), verify=False)
    eq_(r.status_code, 401)

def test_correct_http_status_with_bad_request():
    r = requests.get(nacoma_api_endpoint + 'some-junk-here', auth=('monitor', 'monitor'), verify=False)
    eq_(r.status_code, 404)

def test_auth_method_can_be_disabled():
    old_config = open('/etc/op5/http_api.yml').read()
    locked_down_config = """
auth_method:
  ninja: false
  basic_auth: false
"""
    open('/etc/op5/http_api.yml', 'w').write(locked_down_config)

    for params in [
            {'auth_method': 'basic_auth'},
            {'auth_method': 'ninja'},
            {}
            ]:
        r = requests.get(nacoma_api_endpoint, auth=('monitor', 'monitor'), params=params, verify=False)
        eq_(r.status_code, 403, "Got %d:\n%s" % (r.status_code, r.text))
        eq_(r.json()['full_error'], "The API is locked down, all 'auth_method's are disabled through http_api.yml", "Got %d:\n%s" % (r.status_code, r.text))

    only_basic_auth_config = """
auth_method:
  ninja: false
  basic_auth: true
"""
    open('/etc/op5/http_api.yml', 'w').write(only_basic_auth_config)
    r = requests.get(nacoma_api_endpoint, auth=('monitor', 'monitor'), params={'auth_method': 'ninja'}, verify=False)
    eq_(r.status_code, 412, "Got %d:\n%s" % (r.status_code, r.text))
    eq_(r.json()['full_error'], "Given 'auth_method' (ninja) is disabled through http_api.yml, enable it or use another 'auth_method'", "Got %d:\n%s" % (r.status_code, r.text))

    open('/etc/op5/http_api.yml', 'w').write(old_config)

class AuthModuleConfig(object):
    def __init__(self):
        self.old_auth = open('/etc/op5/auth.yml').read()
        self.old_authusers = open('/etc/op5/auth_users.yml').read()
        self.new_auth = """common:
  session_key: auth_user
  default_auth: robocop
  enable_auto_login: false
robocop:
  driver: Default
predator:
  driver: Default"""
        self.new_authusers = """monitor:
  username: monitor
  realname: Monitor Admin
  password: $1$VGn0CdSG$AMJjvHoF8M2nSy8SiPrW70
  modules:
    - Default
    - robocop
    - predator
  groups:
    - admins
  password_algo: crypt"""
    def insert(self):
        """ @TODO: refactor, since this is a bad idea:
        We're not sure that the fixture below contains the latest and
        greatest needed config options.
        We'd really want to say:
            hey config, erase all old stuff, create *these* modules
        but nacoma's authconfig.php is not a decent enough API for this.

        We could either expose it through the HTTP API, which would let us and
        users access and manipulate config remotely, or we could expose
        authconfig.php's logic via cli.

        The point being: this fixture does not include any business logic
        what so ever - the refactoring must allow us, callers, to not know
        anything about the storage format but provide input parameters and
        have those accepted or not (api 101?)
        """

        with open('/etc/op5/auth.yml', 'w') as conf:
            conf.write(self.new_auth)
        with open('/etc/op5/auth_users.yml', 'w') as conf:
            conf.write(self.new_authusers)
    def reset(self):
        with open('/etc/op5/auth.yml', 'w') as conf:
            conf.write(self.old_auth)
        with open('/etc/op5/auth_users.yml', 'w') as conf:
            conf.write(self.old_authusers)

auth_module_config = AuthModuleConfig()
@with_setup(auth_module_config.insert, auth_module_config.reset)
def test_non_default_auth_method():
    r = requests.get(nacoma_api_endpoint+'host', auth=('monitor$robocop', 'monitor'), verify=False)
    eq_(r.status_code, 200, "Got %d:\n%s\n\n/etc/op5/auth.yml:\n%s\n\n/etc/op5/auth_users.yml:\n%s" % (r.status_code, r.text, open('/etc/op5/auth.yml').read(), open('/etc/op5/auth_users.yml').read()))
    r = requests.get(nacoma_api_endpoint+'host', auth=('monitor$invalid_auth_module', 'monitor'), verify=False)
    eq_(r.status_code, 401, "Got %d:\n%s\n\n/etc/op5/auth.yml:\n%s\n\n/etc/op5/auth_users.yml:\n%s" % (r.status_code, r.text, open('/etc/op5/auth.yml').read(), open('/etc/op5/auth_users.yml').read()))
    r = requests.get(nacoma_api_endpoint+'host', auth=('monitor$predator', 'monitor'), verify=False)
    eq_(r.status_code, 200, "Got %d:\n%s\n\n/etc/op5/auth.yml:\n%s\n\n/etc/op5/auth_users.yml:\n%s" % (r.status_code, r.text, open('/etc/op5/auth.yml').read(), open('/etc/op5/auth_users.yml').read()))
    r = requests.get(nacoma_api_endpoint+'host', auth=('monitor', 'monitor'), verify=False)
    eq_(r.status_code, 200, "Got %d:\n%s\n\n/etc/op5/auth.yml:\n%s\n\n/etc/op5/auth_users.yml:\n%s" % (r.status_code, r.text, open('/etc/op5/auth.yml').read(), open('/etc/op5/auth_users.yml').read()))

# @TODO: implement and respect 'auth_module' as a header, complementing the 'user$auth_module' syntax
