from op5helpers import *
import os
import subprocess

api_endpoint = 'https://'+os.environ.get('MONITOR_SERVER', 'localhost')+'/api/'

op5_license = '/etc/op5license/op5license.xml'
op5_license_blaha = op5_license + '.blaha'

if "check_output" not in dir(subprocess):
    def f(*popenargs, **kwargs):
        """check_output() was introduced in 2.7, copy its (python) source from
        http://hg.python.org/cpython/file/d37f963394aa/Lib/subprocess.py#l544"""
        if 'stdout' in kwargs:
            raise ValueError('stdout argument not allowed, it will be overridden.')
        process = subprocess.Popen(stdout=subprocess.PIPE, *popenargs, **kwargs)
        output, unused_err = process.communicate()
        retcode = process.poll()
        if retcode:
            cmd = kwargs.get("args")
            if cmd is None:
                cmd = popenargs[0]
            raise subprocess.CalledProcessError(retcode, cmd)
        return output
    subprocess.check_output = f

def _std(**kwargs):
    r = requests.get(api_endpoint + kwargs.get('url', ''), auth=kwargs.get('user', authed_user), verify=False)
    try:
        r.json()
    except ValueError:
        ok_(False, "Expected valid JSON, got %d:\n%s" % (r.status_code, r.text))
    code = kwargs.get('code', 200)
    eq_(code, r.status_code, "Should have got status code %d, got %d for URI:\n%s\nbody:\n%s" % (code, r.status_code, r.url, r.text))
    return r

def remove_license():
    os.rename(op5_license, op5_license_blaha)
    # let's also fake a really old report date so that the license is invalid
    # ... yes, this is ugly because we rely on that we "know" what the current
    # way of mocking a too-old license is
    subprocess.check_output(['mysql', 'merlin', '-e', "INSERT INTO report_data (timestamp, event_type, flags, attrib, host_name, service_description, state, hard, retry, downtime_depth, output, long_output) VALUES (1325030554, 100, NULL, NULL, 'just_a_fake', 'aw_yeah', 0, 0, 0, NULL, NULL, NULL)"])

def put_license_back():
    os.rename(op5_license_blaha, op5_license)
    subprocess.check_output(['mysql', 'merlin', '-e', 'DELETE FROM report_data WHERE timestamp = 1325030554 AND host_name = "just_a_fake" AND service_description = "aw_yeah"'])

def adjust_license(relative_hosts=0, *args):
    total_hosts_in_config = len(_std(url='status/host').json())
    os.rename(op5_license, op5_license_blaha)
    new_hosts = total_hosts_in_config + relative_hosts
    args = ['/opt/op5sys/lib/mock_license.php', 'monitorHosts', str(new_hosts)] + list(args)
    subprocess.check_output(args)

def too_small_license():
    adjust_license(-1, 'MonitorLicenseType', 'free')

def one_free_host_in_license():
    adjust_license(1)

@with_setup(remove_license, put_license_back)
def test_expired_license():
    r = _std(code=403)
    ok_('Your license is invalid' in r.text, r.text)

@with_setup(too_small_license, put_license_back)
def test_too_many_hosts_in_config_as_valid_user():
    r = _std(code=200)
    # we should never lock down the API because there are too many
    # hosts in the config. The behavior should always mirror Ninja's

@with_setup(too_small_license, put_license_back)
def test_too_many_hosts_in_config_as_invalid_user():
    r = _std(code=401, user=('pirate', 'swanson'))
    ok_('Unauthorized' in r.text, r.text)
    ok_('You need to login to access this page' in r.text, r.text)

@with_setup(one_free_host_in_license, put_license_back)
def test_save_more_hosts_than_allowed():
    # we should be able to add one host...
    name = 'gayngs'
    host = Crud('host', name)
    host.post({
        'host_name': name,
        'alias': name,
        'address': name
        })

    # ...but this second one should be one too many...
    name2 = 'ludovico einaudi'
    host2 = Crud('host', name2)
    host2.post({
        'host_name': name2,
        'alias': name2,
        'address': name2
        })

    # the actual test
    r = requests.post(api_endpoint + 'config/change', auth=authed_user, verify=False)
    eq_(403, r.status_code, "Should not be able to save more hosts than allowed in configuration, got %d:\n%s" % (r.status_code, r.text))
    ok_('Your usage for monitor is' in r.text, r.text)
    ok_('Forbidden' in r.text, r.text)

    host2.delete()
    host2.verify_gone()

    # just make sure that we actually can save a host
    Change.post()

    # cleanup of the host that we could save:
    host.delete()
    host.verify_gone()
    Change.post()
