from op5helpers import *
import copy, time

standard_params = {
    "auth": ('monitor', 'monitor'),
    "verify": False
}
sections = [u"event"]

# Tip, for python noobs:
# If you try section("yoyo", {"content-type": "application/xml"}) you
# will notice a cryptic error message that means you should name your
# **extra arguments: section("yoyo", headers={"content-type": "application/xml"})

def section(name=None, **extra):
    """
    Helper for requesting a certain URI

    return requests.Response
    """
    params = copy.deepcopy(standard_params) # else we'd be updating by reference
    params.update(extra)
    url = api_endpoint + "api/report"
    if name:
        url += "/" + name
    r = requests.get(url, **params)
    if 'json' in r.headers.get('content-type', "").lower():
        try:
            _ = r.json()
        except ValueError:
            ok_(False, '(%d) Could not get JSON out of:\n%s' % (r.status_code, r.text))
    return r

log("Testing against: "+api_endpoint)

def test_no_std_auth_with_bad_credentials():
    r = section("event", auth=('p_diddys_daddy', 'puppylove'))
    eq_(r.status_code, 401, str(r.status_code)+": "+r.text)

def test_invalid_section():
    r = section("/jallah_akbar")
    eq_(r.status_code, 400, str(r.status_code)+": "+r.text)

def test_can_std_auth():
    r = section()
    eq_(r.status_code, 200, str(r.status_code)+": "+r.text)

def test_event_needs_options():
    r = section("event")
    eq_(r.json()["error"], "No options provided", str(r.status_code)+": "+r.text)
    eq_(r.status_code, 400, str(r.status_code)+": "+r.text)

@import_log("simple")
def test_event_exposed_properties():
    """ This is a regression test. Changes here should only occur in major versions """

    # all events within timestamps
    r = section('event', params={})
    r = section("event", params={'start_time': 1364906100, 'end_time': 1364906105, "limit": 5})
    eq_(r.status_code, 200, str(r.status_code)+": "+r.text)
    eq_(len(r.json()['data']), 5, json.dumps(r.json()))
    container_keys = set([u'resource', u'start_time', u'limit', u'end_time', u'offset', u'generated_at', u'data'])
    container_key_diff =  container_keys ^ set(r.json().keys())
    eq_(container_key_diff, set(), "Uh oh, you broke the event API by not sending the same keys as specified, diff:\n" + str(container_key_diff) + "\n" + r.text)

    # subset of all events within timestamps, check for 'next_resource'
    r = section("event", params={'start_time': 1364906100, 'end_time': 1364906105, "limit": 1})
    eq_(len(r.json()['data']), 1, json.dumps(r.json()))
    container_keys = set([u'resource', u'start_time', u'limit', u'end_time', u'offset', u'generated_at', u'data', u'next_resource'])
    container_key_diff =  container_keys ^ set(r.json().keys())
    eq_(container_key_diff, set(), "Uh oh, you broke the event API by not sending the same keys as specified, diff: " + str(container_key_diff))
    ok_('format=json' not in r.json()['next_resource'], 'Should not have included the requested format in the next_resource')
    ok_('format=json' not in r.json()['resource'], 'Should not have included the requested format in the resource')

    event_key_diff = set([u'retry', u'event_type', u'timestamp', u'hard', u'state', u'host_name', u'output', u'service_description', u'in_scheduled_downtime']) ^ set(r.json()['data'][0].keys())
    eq_(event_key_diff, set(), "Uh oh, you broke the event API by not sending the same keys as specified, diff:\n" + str(event_key_diff) + "\n" + r.text)

@import_log("simple")
def test_event_properties_input_translation():
    """ Make sure we can 'deselect' host alert types by specifying the
    string value 'service' to the option 'alert_types' """
    r = section('event', params={'start_time': 1364906100, 'end_time': 1364906105, 'alert_types': 'service'})
    eq_(r.status_code, 200, str(r.status_code)+": "+r.text)
    eq_(len(r.json()['data']), 3, r.text)

@import_log("simple")
def test_event_properties_output_translation():
    r = section('event', params={'start_time': 1364906100, 'end_time': 1364906105})
    eq_(r.status_code, 200, str(r.status_code)+": "+r.text)
    eq_(len(r.json()['data']), 5, r.text)

    for entry in r.json()['data']:
        # cheap way of saying that "Nagios' properties are translated to strings"
        ok_(not is_number(entry['event_type']), "%s should not be a number" % entry['event_type'])
        ok_(not is_number(entry['state']), "%s should not be a number" % entry['state'])
        ok_(is_number(entry['in_scheduled_downtime']), "%s should not be a number" % entry['state'])

@import_log("simple")
def test_event_pagination():
    first = section('event', params={'start_time': 1364906100, 'end_time': 1364906101, 'limit': 1})
    eq_(first.status_code, 200)

    eq_(len(first.json()['data']), 1, first.text)
    eq_(first.json()['data'][0]['output'], 'OK - laa-laa', first.json()['data'][0])
    # @todo add tests for valid url + valid parameters in the resource keys
    ok_('next_resource' in first.json(), first.text)
    eq_(first.json()['limit'], 1, first.text)
    eq_(first.json()['offset'], 0, first.text)
    eq_(first.json()['start_time'], str(1364906100), json.dumps(first.json()))
    eq_(first.json()['end_time'], str(1364906101), first.text)
    ok_('generated_at' in first.json(), first.text)
    first_data = first.json()['data'][0]

    second = section('event', params={'start_time': 1364906100, 'end_time': 1364906101, 'limit': 1, 'offset': 1})
    eq_(second.status_code, 200, str(second.status_code)+": "+second.text)
    eq_(len(second.json()['data']), 1, second.text)
    eq_(second.json()['data'][0]['output'], 'OK - Sven Melander', second.json()['data'][0])
    eq_(second.json()['limit'], 1, first.text)
    eq_(second.json()['offset'], 1, first.text)
    eq_(second.json()['start_time'], str(1364906100), first.text)
    eq_(second.json()['end_time'], str(1364906101), first.text)

    second_data = second.json()['data'][0]
    ok_(cmp(first_data, second_data) != 0, "first: \n"+str(first_data)+"\n\nsecond:\n"+str(second_data))

@import_log("simple")
def test_event_traversing():
    """ Almost like the pagination test but a complete POC of how to traverse a possibly large result set """
    sys.path.insert(0, os.path.abspath("examples"))
    sys.path.insert(0, os.path.abspath("../api/examples"))
    try:
        import traverse
    except ImportError as e:
        log(e)
        assert False
    traverse.auth = standard_params['auth']
    traverse.params['report_period'] = 'custom'
    traverse.params['start_time'] = 1364906100
    traverse.api_server = api_endpoint
    result = traverse.traverse()
    ok_(len(result))

@import_log("simple")
def test_timeperiod():
    r = section('event', params={'report_period': 'last24hours', 'limit': 1})
    now = int(r.json()['generated_at'])
    before_last_24_hours = now - (24 * 60 * 60) - 1
    ok_(abs(before_last_24_hours - int(r.json()['start_time'])) < 20, "start time and 24 hours ago should not diff more than 20 seconds")
    ok_(abs(now - int(r.json()['end_time'])) < 20, "end time and now should not diff more than 20 seconds")

def test_index_defaults_to_help():
    r = section(params={"format": "html"})
    eq_(r.status_code, 200)
    ok_("This is the REST-API" in r.content)

def test_help():
    r = requests.get(api_endpoint+"api/help/report", **standard_params)
    eq_(r.status_code, 200, str(r.status_code)+": "+r.text)
    r = requests.get(api_endpoint+"api/help/report/event", **standard_params)
    eq_(r.status_code, 200, str(r.status_code)+": "+r.text)
    r = requests.get(api_endpoint+"api/help/report/jsdahf", **standard_params)
    ok_(r.status_code != 200, str(r.status_code)+": "+r.text)

def test_content_negotiation():
    formats = {
        "xml": "application/xml; charset=utf-8",
        "json": "application/json; charset=utf-8",
        "html": "text/html; charset=utf-8"
    }
    for f in formats:
        content_type = formats[f]
        r = section("event", headers={'accept': content_type.split(";")[0]}, params={'limit': 1})
        eq_(r.headers["content-type"], content_type, str(r.status_code)+": "+r.text+"\n\nheaders:\n"+str(r.headers))

        r = section("event", params={"format": f})
        eq_(r.headers["content-type"], content_type, str(r.status_code)+": "+r.text+"\n\nheaders:\n"+str(r.headers))

def test_event_defaults_to_json():
    r = section("event")
    eq_(r.headers["content-type"], "application/json; charset=utf-8", str(r.status_code)+": "+r.text+"\n\nheaders:\n"+str(r.headers))

@import_log("simple")
def test_restrict_object_selection():
    params = {
        "start_time": 1364906100,
        "end_time": 1364906101,
        "host_name": ['printer1']
    }
    authed = section(
        "event",
        params=params
    )
    eq_(len(authed.json()['data']), 1, "Should have filtered out not-asked-for events:\n"+str(authed.json()))

def test_fails_without_params():
    r = section("event")
    eq_(r.status_code, 400)
    eq_(r.json()['error'], "No options provided")
    eq_(r.json()['full_error'], "No options provided")

# @todo test the report_api access right

@import_log("simple")
def test_limited_user():
    params = {
        "start_time": 1364906100,
        "end_time": 1364906106
    }
    authed = section(
        "event",
        params=params
    )
    group_name = 'shadonda'
    username = "latisha"

    ok_(authed.json() and 'data' in authed.json(), "No JSON?\n"+authed.text) # useful for debugging via var_dump in ninja
    eq_(len(authed.json()['data']), 5, "We should have the same amount of events as defined in simple.log, got " + str(len(authed.json()['data'])) + ":\n" + str(authed.json()))

    usergroup = Crud('usergroup', group_name)
    usergroup.post({
        'name': group_name,
        'access_rights': ['host_view_contact', 'api_report']
    })
    user = Crud('user', username)
    r = user.post({
        'username': username,
        'password': username,
        'modules': ['Default'],
        'groups': [group_name]
    }, cmp=False)
    contact = Crud('contact', username)
    contact.post({
        'contact_name': username,
        'alias': username
    }, cmp=False)
    host = Crud('host', 'printer1', clean=False)
    host.patch({
        'contacts': [username]
    })
    Change.post()

    not_authed = section(
        "event",
        params=params,
        auth=(username, username)
    )
    eq_(not_authed.status_code, 200, str(not_authed.status_code)+": "+not_authed.text)
    ok_(not_authed.json() and 'data' in not_authed.json(), "No JSON?\n"+not_authed.text) # useful for debugging via var_dump in ninja
    eq_(len(not_authed.json()['data']), 1, "We got " + str(len(authed.json()['data'])) + " events instead of only 1, as defined in simple.log. Response:\n" + json.dumps(authed.json()))
    Crud('user', username)
    Crud('usergroup', group_name)
    Crud('contact', username)
    Change.post()

@import_log("simple")
def test_timestamps_are_inclusive():
    for param_couple in [
            {'start_time': 1364906099, 'end_time': 1364906100},
            {'start_time': 1364906100, 'end_time': 1364906100},
            {'start_time': 1364906104, 'end_time': 1364906105}
            ]:
        r = section('event', params=param_couple)
        eq_(200, r.status_code, str(r.status_code) + ":\n" + r.text)
        eq_(1, len(r.json()['data']), "All events should be in the given timeframe:\n" + r.text)

def test_timestamp_validation():
    r = section('event', params={'start_time': 1364906100, 'end_time': 136490099})
    eq_(400, r.status_code, str(r.status_code) + ":\n" + r.text)

@import_log("simple")
def test_limit_by_object_name():
    r = section('event', params={'start_time': 1364906101, 'host_name': ['printer1']})
    eq_(200, r.status_code, str(r.status_code) + ":\n" + r.text)
    evs = len(r.json()['data'])
    eq_(1, evs, "Got %d events, wanted 1:\n%s" % (evs, r.text))

    # let's try to make the API translate a string to an array as well
    r = section('event', params={'start_time': 1364906101, 'host_name': 'printer1'})
    eq_(200, r.status_code, str(r.status_code) + ":\n" + r.text)
    evs = len(r.json()['data'])
    eq_(1, evs, "Got %d events, wanted 1:\n%s" % (evs, r.text))
